package com.github.itzilly.sbt.features

import com.github.itzilly.sbt.renderer.RenderFuncs
import com.github.itzilly.sbt.util.Chatterbox
import com.github.itzilly.sbt.util.MessageScheduler
import com.github.itzilly.sbt.util.SimpleChatMsg
import net.minecraft.block.Block
import net.minecraft.block.state.IBlockState
import net.minecraft.client.Minecraft
import net.minecraft.client.network.NetHandlerPlayClient
import net.minecraft.init.Blocks
import net.minecraft.util.BlockPos
import net.minecraft.world.World
import net.minecraft.world.chunk.Chunk
import net.minecraftforge.client.event.RenderWorldLastEvent
import net.minecraftforge.event.world.ChunkEvent
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent


object GrottoFinder {
    private val targetBlock: Block = Blocks.stained_glass
    private val targetMeta: Int = 2 // Magenta stained glass

    private var magentaGlassList: ArrayList<ChestNode> = ArrayList()

    private val minBounds = BlockPos(201, 30, 201)
    private val maxBounds = BlockPos(824, 189, 824)

    var isEnabled: Boolean = true

    fun clear() {
        magentaGlassList.clear()
    }

    fun checkWorldForMagentaStainedGlass(world: World) {
        for (chunkX in minBounds.x / 16..maxBounds.x / 16) {
            for (chunkZ in minBounds.z / 16..maxBounds.z / 16) {
                val chunk = world.getChunkFromChunkCoords(chunkX, chunkZ)

                // Iterate through the chunk's blocks within the world bounds
                for (x in 0..15) {
                    for (y in minBounds.y..maxBounds.y) {
                        for (z in 0..15) {
                            val worldX = chunk.xPosition * 16 + x
                            val worldZ = chunk.zPosition * 16 + z

                            // Ensure the block is within the specified bounds
                            if (worldX in minBounds.x..maxBounds.x && worldZ in minBounds.z..maxBounds.z) {
                                val blockPos = BlockPos(worldX, y, worldZ)
                                val blockState = chunk.getBlockState(blockPos)
                                val block = blockState.block
                                val meta = block.getMetaFromState(blockState)

                                if (block === Blocks.stained_glass && meta == 2) {
                                    if (!magentaNodeExists(blockPos.x, blockPos.y, blockPos.z)) {
                                        magentaGlassList.add(ChestNode(blockPos.x, blockPos.y, blockPos.z))
                                    }
                                    println("Found magenta stained glass at $blockPos")
                                    val msg = SimpleChatMsg("Found magenta stained glass at $blockPos").light_purple()
                                    Chatterbox.say(msg)
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    @SubscribeEvent
    fun renderBlockOverlay(event: RenderWorldLastEvent) {
        val player = Minecraft.getMinecraft().thePlayer

        for (glass: ChestNode in magentaGlassList) {
            val pos = BlockPos(glass.x, glass.y, glass.z)
            RenderFuncs.drawBlockOutline(player, pos, event.partialTicks)
            RenderFuncs.drawBlockLine(pos, event.partialTicks)
        }
    }

    private fun magentaNodeExists(x: Int, y: Int, z: Int): Boolean {
        return magentaGlassList.any { it.x == x && it.y == y && it.z == z }
    }

}


// World Bounds 824, 189, 824 201, 30, 201