package com.github.itzilly.sbt.features.gardener

import com.github.itzilly.sbt.util.Chatterbox
import net.minecraft.client.gui.*

class MacroMakerGui : GuiScreen() {
    private var selectedTaskIndex: Int = -1
    private var draggedTaskIndex: Int = -1
    private var dragOffsetY: Int = 0

    private val macro = Macro("Example Macro")

    enum class ButtonId(val id: Int) {
        SAVE_MACRO(0),
        CANCEL(1)
    }

    override fun initGui() {
        this.buttonList.clear()
        buttonList.add(GuiButton(ButtonId.SAVE_MACRO.id, width / 2 - 100, height - 40, 90, 20, "Save Macro"))
        buttonList.add(GuiButton(ButtonId.CANCEL.id, width / 2 + 10, height - 40, 90, 20, "Cancel"))

        macro.tasks.add(MoveTask("W", true))
        macro.tasks.add(MouseTask("Left Click"))
        macro.tasks.add(ChatCommandTask("/spawn"))
    }

    override fun actionPerformed(button: GuiButton) {
        when (button.id) {
            ButtonId.SAVE_MACRO.id -> {
                Chatterbox.say("Saved macro!")
                Close()
            }
            ButtonId.CANCEL.id -> Close()
        }
    }

    override fun drawScreen(mouseX: Int, mouseY: Int, partialTicks: Float) {
        drawDefaultBackground()

        val thirdWidth = width / 3
        drawRect(0, 0, thirdWidth, height, 0xFF202020.toInt())
        drawRect(thirdWidth, 0, thirdWidth * 2, height, 0xFF303030.toInt())
        drawRect(thirdWidth * 2, 0, width, height, 0xFF202020.toInt())

        drawCenteredString(fontRendererObj, "Macro Maker", width / 2, 10, 0xFFFFFF)

        drawLeftPanel(mouseX, mouseY)
        drawMiddlePanel(mouseX, mouseY)
        drawRightPanel()

        super.drawScreen(mouseX, mouseY, partialTicks)
    }

    private fun drawLeftPanel(mouseX: Int, mouseY: Int) {
        drawString(fontRendererObj, "Editing: ${macro.name}", 10, 20, 0xFFFFFF)
        drawString(fontRendererObj, "Add Task:", 10, 40, 0xAAAAAA)

        val taskButtons = listOf(
            Triple("+ Move", 60, { macro.tasks.add(MoveTask("W", false)) }),
            Triple("+ Mouse", 80, { macro.tasks.add(MouseTask("Right Click")) }),
            Triple("+ Command", 100, { macro.tasks.add(ChatCommandTask("/say Hello")) })
        )

        for ((label, y, _) in taskButtons) {
            drawString(fontRendererObj, label, 10, y, 0x00FF00)
        }

        if (mouseClickedX in 10..100 && mouseClickedY != -1) {
            for ((_, y, action) in taskButtons) {
                if (mouseClickedY in y..(y + 10)) {
                    action.invoke()
                    break
                }
            }
            mouseClickedX = -1
            mouseClickedY = -1
        }
    }

    private fun drawMiddlePanel(mouseX: Int, mouseY: Int) {
        val thirdWidth = width / 3
        val taskStartY = 40
        val taskHeight = 20

        for ((index, task) in macro.tasks.withIndex()) {
            val y = taskStartY + index * (taskHeight + 5)
            if (draggedTaskIndex == index) continue // skip rendering the dragged one
            val color = if (index == selectedTaskIndex) 0xFF4444 else 0xFFFFFF
            drawRect(thirdWidth + 10, y, thirdWidth * 2 - 10, y + taskHeight, 0xFF404040.toInt())
            drawString(fontRendererObj, "${task.name} - ${task.getPreview()}", thirdWidth + 15, y + 6, color)
        }

        // Draw the dragged task following the mouse
        if (draggedTaskIndex != -1) {
            val task = macro.tasks[draggedTaskIndex]
            val y = mouseY - dragOffsetY
            drawRect(thirdWidth + 10, y, thirdWidth * 2 - 10, y + taskHeight, 0xFF606060.toInt())
            drawString(fontRendererObj, "${task.name} - ${task.getPreview()}", thirdWidth + 15, y + 6, 0xFFFF00)
        }
    }

    private fun drawRightPanel() {
        val selected = macro.tasks.getOrNull(selectedTaskIndex) ?: return
        val startX = width - 180
        val startY = 20
        val boxWidth = 160
        val boxHeight = 80

        // Box with outline
        drawRect(startX - 2, startY - 2, startX + boxWidth + 2, startY + boxHeight + 2, 0xFFFFFFFF.toInt())
        drawRect(startX, startY, startX + boxWidth, startY + boxHeight, 0xFF101010.toInt())

        drawString(fontRendererObj, "Task Properties", startX + 5, startY + 5, 0xFFFFFF)

        when (selected) {
            is MoveTask -> {
                drawString(fontRendererObj, "Direction: ${selected.direction}", startX + 5, startY + 25, 0xCCCCCC)
                drawString(fontRendererObj, "Hold: ${selected.hold}", startX + 5, startY + 40, 0xCCCCCC)
            }
            is MouseTask -> {
                drawString(fontRendererObj, "Action: ${selected.action}", startX + 5, startY + 25, 0xCCCCCC)
            }
            is ChatCommandTask -> {
                drawString(fontRendererObj, "Command:", startX + 5, startY + 25, 0xCCCCCC)
                drawString(fontRendererObj, selected.command, startX + 5, startY + 40, 0x00FFAA)
            }
        }
    }

    private var mouseClickedX = -1
    private var mouseClickedY = -1

    override fun mouseClicked(mouseX: Int, mouseY: Int, mouseButton: Int) {
        mouseClickedX = mouseX
        mouseClickedY = mouseY

        val thirdWidth = width / 3
        val taskStartY = 40
        val taskHeight = 25

        for ((index, _) in macro.tasks.withIndex()) {
            val y = taskStartY + index * taskHeight
            if (mouseX in (thirdWidth + 10)..(thirdWidth * 2 - 10) && mouseY in y..(y + 20)) {
                selectedTaskIndex = index
                draggedTaskIndex = index
                dragOffsetY = mouseY - y
            }
        }

        super.mouseClicked(mouseX, mouseY, mouseButton)
    }

    override fun mouseClickMove(mouseX: Int, mouseY: Int, clickedMouseButton: Int, timeSinceLastClick: Long) {
        super.mouseClickMove(mouseX, mouseY, clickedMouseButton, timeSinceLastClick)
    }

    override fun mouseReleased(mouseX: Int, mouseY: Int, state: Int) {
        if (draggedTaskIndex != -1) {
            val thirdWidth = width / 3
            val taskStartY = 40
            val taskHeight = 25
            val newIndex = ((mouseY - taskStartY) / taskHeight).coerceIn(0, macro.tasks.size - 1)
            if (newIndex != draggedTaskIndex) {
                val task = macro.tasks.removeAt(draggedTaskIndex)
                macro.tasks.add(newIndex, task)
                selectedTaskIndex = newIndex
            }
            draggedTaskIndex = -1
        }

        super.mouseReleased(mouseX, mouseY, state)
    }

    fun Close() {
        mc.displayGuiScreen(null)
    }
}

sealed class MacroTask(val name: String) {
    abstract fun getPreview(): String
}

class MoveTask(var direction: String, var hold: Boolean) : MacroTask("Move") {
    override fun getPreview(): String = if (hold) "Hold $direction" else "Tap $direction"
}

class MouseTask(val action: String) : MacroTask("Mouse") {
    override fun getPreview(): String = action
}

class ChatCommandTask(var command: String) : MacroTask("Command") {
    override fun getPreview(): String = command
}

data class Macro(
    val name: String,
    val tasks: MutableList<MacroTask> = mutableListOf()
)
